// Totem Library include
#include <Wire.h>

#include "config.h"


#define COMMAND_LED_OFF     0x00
#define COMMAND_LED_ON      0x01
#define COMMAND_GET_VALUE        0x05
#define COMMAND_NOTHING_NEW   0x99

const byte qwiicAddress = 0x28;     //Default Address
Module22 sensor;

AdafruitIO_Feed *soilHumidity = io.feed("soil_humid");
AdafruitIO_Feed *airHumidity = io.feed("air_humid");
AdafruitIO_Feed *airTemp = io.feed("temperature");
AdafruitIO_Feed *fanSpeed = io.feed("fan_speed");
#define IO_LOOP_DELAY 15000
unsigned long lastUpdate;

void ledOn() {
  Wire.beginTransmission(qwiicAddress);
  Wire.write(COMMAND_LED_ON);
  Wire.endTransmission();
}

void ledOff() {
  Wire.beginTransmission(qwiicAddress);
  Wire.write(COMMAND_LED_OFF);
  Wire.endTransmission();
}


// testForConnectivity() checks for an ACK from an Sensor. If no ACK
// program freezes and notifies user.
void testForConnectivity() {
  Wire.beginTransmission(qwiicAddress);
  //check here for an ACK from the slave, if no ACK don't allow change?
  if (Wire.endTransmission() != 0) {
    Serial.println("Check connections. No slave attached.");
    while (1);
  }
}

// LED is off, and a -1 if an error occurred.
uint16_t get_value() {
  uint16_t ADC_VALUE;
    
  Wire.beginTransmission(qwiicAddress);
  Wire.write(COMMAND_GET_VALUE); // command for status
  Wire.endTransmission();    // stop transmitting //this looks like it was essential.

  Wire.requestFrom(qwiicAddress, 2);    // request 1 bytes from slave device qwiicAddress

  while (Wire.available()) { // slave may send less than requested
  uint8_t ADC_VALUE_L = Wire.read(); 
  uint8_t ADC_VALUE_H = Wire.read();

  ADC_VALUE=ADC_VALUE_H;
  ADC_VALUE<<=8;
  ADC_VALUE|=ADC_VALUE_L;
  }
  uint16_t x=Wire.read(); 

  return ADC_VALUE;
}

void fanSpeedHandler(AdafruitIO_Data *data) {

  Serial.print("received <- ");

  // since we are using the same function to handle
  // messages for two feeds, we can use feedName() in
  // order to find out which feed the message came from.
  Serial.print(data->feedName());
  Serial.print(" ");

  // print out the received count or counter-two value
  Serial.println(data->value());

  
  int value = atoi(data->value());
  Serial.println(value);
  X4.dcD.power(-value); // Spin at 10% power

}
// Arduino setup function.
void setup() {
  Serial.begin(9600);
    // Initialize X4 module

    Wire.begin(GPIOA, GPIOB); // Set SDA and SCL pins
    testForConnectivity();
    ledOn();

      // connect to io.adafruit.com
    io.connect();

  // attach message handler for the counter feed.
  fanSpeed->onMessage(fanSpeedHandler);
    
}
// Arduino loop function
void loop() {
  io.run();
  if (millis() > (lastUpdate + IO_LOOP_DELAY)) {
    uint16_t soilHumid = get_value();
    soilHumidity->save(soilHumid);
    airTemp->save(sensor.temp.getC());
    airHumidity->save(sensor.humidity.get());
    lastUpdate = millis();
  }
}
